#include <windows.h>
#include <IntLib.h>
#include <CoProcLib.h>
#include <MapRegLib.h>
#include <GPIOLib.h>

// NOTE: We want to interrupt on a GPIO located on Pin 48 of the Lower PlugBoard of the Colibri evaluation board V2.1 (EXT_IO[0])
// SODIMM Pin 135

//TIPs for short a short latency
//-Colibri Tegra, you can deactivate the DFS (Variable Frequency/Voltage switching)
//-Consider to use an ISR (installable interrupt routine)


int WINAPI WinMain(HINSTANCE hInstance,      // handle to current instance
                   HINSTANCE hPrevInstance,  // handle to previous instance
                   LPWSTR lpCmdLine,          // pointer to command line
                   int nCmdShow)             // show state of window
{
	HANDLE	hEvent;
	DWORD	dwEdge, dwGpio;
	DWORD	dwIrq, dwSysIntr;
	PIN_INSTANCE gpio;
 
	//************************************
	// GPIO Specific Section

	//Set SODIMM Pin 135 as GPIO Input
	SetPinAltFn(135,-1,DIR_IN);

	//Get GPIO Number for SODIMM Pin 135
	GetGPIOFromPin(135, FALSE,&gpio);
	dwGpio = gpio.inst1;

	//Get IRQ for this particular GPIO
	dwIrq=GetGPIOIrq(dwGpio);
	if(!dwIrq) goto leave_error; 

	// Set Edge to trigger
	//dwEdge=GPIO_LEVEL_HIGH //level interrupts
	//dwEdge=GPIO_EDGE_RISING | GPIO_EDGE_FALLING; // (rising & falling edge)
	dwEdge=GPIO_EDGE_RISING;	
	
	if(!SetGPIOIrqEdge(dwGpio, dwEdge)) goto leave_error;
 

	//************************************
	// General Interrupt Handling Section	

	// Create an Event to wait on
	hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
	if ( hEvent == NULL) goto leave_error;

	// Get the SYSINTR that corresponds to dwIrq (Irqs corresond to the bit numbers of the PXAx Interrupt resisters)
	dwSysIntr=RequestSysInterrupt(dwIrq);
	if (!dwSysIntr) goto leave_error;

	// Link our Event with the SYSINTR
	if ( !InterruptInitializeCompat(dwSysIntr, hEvent, NULL, 0) ) goto leave_error;


	
	// NOTE: Normally this will be done in a separate Thread

	//Maybe you like to increase the priorty of your interrupt Thread
	CeSetThreadPriority(GetCurrentThread(), 200); 
	while(1)
	{
		// Wait for Event (Interrupt)
		if (WaitForSingleObject(hEvent, INFINITE) == WAIT_OBJECT_0)
		{
			if(MessageBox(NULL, L"Interrupt Event1 Detected, continue Waiting?", L"Interrupt", MB_YESNO) == IDNO) break;
			// NOTE: It's possible that we get the same interrupt twice if the signal is not debounced
			InterruptDoneCompat(dwSysIntr);
		}
	}


	//It is very important to deinitalize the interrupt, otherwise it will not work the next time you start the program
	InterruptDisableCompat(dwSysIntr);
	if (!ReleaseSysIntr(dwSysIntr)) goto leave_error;

	return 0;

leave_error:

	MessageBox(NULL, L"An Error occurred!\nDid you release the interrupt correctly?", L"ERROR", MB_OK);

	return -1;

    


}

